import torch
import torchvision.transforms as transforms
import numpy as np
import cv2
import logging

from .model import Net

class Extractor(object):
    def __init__(self, model_path, use_cuda=True):
        self.net = Net(reid=True)
        self.device = "cuda" if torch.cuda.is_available() and use_cuda else "cpu"
        state_dict = torch.load(model_path, map_location=lambda storage, loc: storage)['net_dict']
        self.net.load_state_dict(state_dict)
        logger = logging.getLogger("root.tracker")
        logger.info("Loading weights from {}... Done!".format(model_path))
        self.net.to(self.device)
        self.size = (64, 128)
        self.norm = transforms.Compose([
            transforms.ToTensor(),
            transforms.Normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225]),
        ])
        


    def _preprocess(self, im_crops):
        """
        TODO:
            1. to float with scale from 0 to 1
            2. resize to (64, 128) as Market1501 dataset did
            3. concatenate to a numpy array
            3. to torch Tensor
            4. normalize
        """
        def _resize(im, size):
            return cv2.resize(im.astype(np.float32)/255., size)

        im_batch = torch.cat([self.norm(_resize(im, self.size)).unsqueeze(0) for im in im_crops], dim=0).float()
        return im_batch


    def __call__(self, im_crops):
        im_batch = self._preprocess(im_crops)
        with torch.no_grad():
            im_batch = im_batch.to(self.device)
            features = self.net(im_batch)
        return features.cpu().numpy()


if __name__ == '__main__':
    img = cv2.imread("demo.jpg")[:,:,(2,1,0)]
    extr = Extractor("checkpoint/ckpt.t7")
    feature = extr(img)
    print(feature.shape)




import numpy as np
import cv2
import datetime
from vhh_od.utils import *
from vhh_od.Shot import Shot
from PIL import Image
import torchvision


class Video(object):
    """
    This class is representing a video. Each instance of this class is holding the properties of one Video.
    """

    def __init__(self):
        """
        Constructor
        """

        #printCustom("create instance of video class ... ", STDOUT_TYPE.INFO);
        self.vidFile = ''
        self.vidName = ""
        self.frame_rate = 10
        self.channels = 0
        self.height = 0
        self.width = 0
        self.format = ''
        self.length = 0
        self.number_of_frames = 0
        self.vid = None
        self.convert_to_gray = False
        self.convert_to_hsv = False
        self.shot_list = []

    def addShotObject(self, shot_obj: Shot):
        self.shot_list.append(shot_obj)

    def load(self, vidFile: str):
        """
        Method to load video file.

        :param vidFile: [required] string representing path to video file
        """

        #print(vidFile)
        printCustom(f"Loading Video \"{vidFile}\"... ", STDOUT_TYPE.INFO)
        self.vidFile = vidFile
        if(self.vidFile == ""):
            #print("A")
            print("ERROR: you must add a video file path!")
            exit(1)
        self.vidName = self.vidFile.split('/')[-1]
        self.vid = cv2.VideoCapture(self.vidFile)

        if(self.vid.isOpened() == False):
            #print("B")
            print(f"ERROR: not able to open video file \"{vidFile}\"!")
            exit(1)

        status, frm = self.vid.read()

        self.channels = frm.shape[2]
        self.height = self.vid.get(cv2.CAP_PROP_FRAME_HEIGHT)
        self.width = self.vid.get(cv2.CAP_PROP_FRAME_WIDTH)
        self.frame_rate = self.vid.get(cv2.CAP_PROP_FPS)
        self.format = self.vid.get(cv2.CAP_PROP_FORMAT)
        self.number_of_frames = self.vid.get(cv2.CAP_PROP_FRAME_COUNT)

        self.vid.release()

    def printVIDInfo(self):
        """
        Method to a print summary of video properties.
        """

        print("---------------------------------")
        print("Video information")
        print("filename: " + str(self.vidFile))
        print("format: " + str(self.format))
        print("fps: " + str(self.frame_rate))
        print("channels: " + str(self.channels))
        print("width: " + str(self.width))
        print("height: " + str(self.height))
        print("nFrames: " + str(self.number_of_frames))
        print("---------------------------------")
        print("<<< Shot list >>>")
        for shot in self.shot_list:
            shot.printShotInfo()

    def getAllFrames(self, preprocess_pytorch=None):
        # read all frames of video
        cap = cv2.VideoCapture(self.vidFile)

        frame_l = []
        frames_orig = []

        cnt = 0
        while (True):
            cnt = cnt + 1
            ret, frame_orig = cap.read()

            # print(cnt)
            # print(ret)
            # print(cap.get(cv2.CAP_PROP_FRAME_COUNT))
            if (ret == True):
                if(preprocess_pytorch is not None):
                    frame = preprocess_pytorch(frame_orig)
                frame_l.append(frame)  # .transpose((2,0,1)

                # Deep Sort Tracker needs original image in RGB Space
                frames_orig.append(cv2.cvtColor(frame_orig, cv2.COLOR_BGR2RGB))
            else:
                break
        cap.release()

        if (preprocess_pytorch is not None):
            all_tensors_l = torch.stack(frame_l)
            return {"Tensors": all_tensors_l, "Images" : frames_orig}

    # generator for loading Video Frame by Frame
    def loadVideoByFrame(self, preprocess_pytorch=None):

        cap = cv2.VideoCapture(self.vidFile)

        cnt = 0
        while (True):
            cnt = cnt + 1
            ret, frame_orig = cap.read()

            if (ret == True):
                if(preprocess_pytorch is not None):
                    frame_processed = preprocess_pytorch(frame_orig)
                else:
                    frame_processed = None
                frame_orig = cv2.cvtColor(frame_orig, cv2.COLOR_BGR2RGB)
                yield {"Tensors": frame_processed, "Images" : frame_orig}
            else:
                break
        cap.release()

    def getFrame(self, frame_id):
        """
        Method to get one frame of a video on a specified position.

        :param frame_id: [required] integer value with valid frame index
        :return: numpy frame (WxHx3)
        """

        self.vid.open(self.vidFile)
        if(frame_id >= self.number_of_frames):
            print("ERROR: frame idx out of range!")
            return []

        #print("Read frame with id: " + str(frame_id));
        time_stamp_start = datetime.datetime.now().timestamp()

        self.vid.set(cv2.CAP_PROP_POS_FRAMES, frame_id)
        status, frame_np = self.vid.read()
        self.vid.release()

        if(status == True):
            if(self.convert_to_gray == True):
                frame_np = cv2.cvtColor(frame_np, cv2.COLOR_BGR2GRAY)
                #print(frame_gray_np.shape);
            if (self.convert_to_hsv == True):
                frame_np = cv2.cvtColor(frame_np, cv2.COLOR_BGR2HSV)
                h, s, v = cv2.split(frame_np)

        time_stamp_end = datetime.datetime.now().timestamp()
        time_diff = time_stamp_end - time_stamp_start
        #print("time: " + str(round(time_diff, 4)) + " sec")

        return frame_np

    def getFramesByShots_NEW(self, preprocess_pytorch=None, max_frames_per_return  = 2000):
        """
        Returns Video Shot by Shot
        If a single shot is longer than X frames, it will split the shot into frames of length at most X. 
        """
        # initialize video capture
        cap = cv2.VideoCapture(self.vidFile)

        frame_number = 0
        for shot in self.shot_list:

            frame_l = []
            frames_orig = []

            sid = int(shot.sid)
            start_idx = int(shot.start_pos)
            stop_idx = int(shot.end_pos)
            shot_is_not_over = True

            while shot_is_not_over:
                # print(f"Retrieving Frames for Shot {sid} (frames {frame_number} to {stop_idx})...")
                while frame_number < stop_idx and len(frame_l) < max_frames_per_return:
                    
                    # read next frame
                    success, image = cap.read()
                    frame_number = frame_number + 1
                    #print(frame_number)

                    # if(start_idx == stop_idx):
                    #    cv2.imshow("frame", image)
                    #    k = cv2.waitKey()

                    # skip to start position (for gradual cuts)
                    if frame_number < start_idx:
                        # print(frame_number)
                        continue

                    if success == True:
                        # if ( (frame_number >= start_idx and frame_number <= stop_idx) or (start_idx == stop_idx) ):
                        if (preprocess_pytorch != None):
                            frames_orig.append(image)
                            image = preprocess_pytorch(image)
                            frame_l.append(image)
                        else:
                            frames_orig.append(image)
                    else:
                        break

                if frame_l != [] and preprocess_pytorch is not None:
                    all_tensors_l = torch.stack(frame_l)
                    yield {"Tensors": all_tensors_l, "Images": np.array(frames_orig), "ShotInfo": shot}
                else:
                    yield {"Tensors": None, "Images": np.array(frames_orig), "ShotInfo": shot}

                # End the shot if the current frame is the last frame (or an even later frame)
                if not frame_number < stop_idx:
                    shot_is_not_over = False
                # If the shot is not over, prepare to return more frames in this shot   
                else:
                    frame_l = []
                    frames_orig = []

        cap.release()

    # Returns Video Shot by Shot
    def getFramesByShots(self, preprocess_pytorch=None):
        # initialize video capture
        cap = cv2.VideoCapture(self.vidFile)
        cnt = 0

        for shot in self.shot_list:

            frame_l = []
            frames_orig = []
            start_pos = shot.start_pos
            end_pos = shot.end_pos

            #print(f"Retrieving Frames for Shot {shot.sid} (frames {cnt} to {end_pos})...")

            while cnt <= end_pos:
                cnt = cnt + 1
                ret, frame_orig = cap.read()

                # skip to start position (for gradual cuts)
                if cnt < start_pos:
                    continue

                if ret == True:
                    if preprocess_pytorch is not None:
                        frame = preprocess_pytorch(frame_orig)
                        frame_l.append(frame)

                    # Deep Sort Tracker needs original image in RGB Space
                    frames_orig.append(cv2.cvtColor(frame_orig, cv2.COLOR_BGR2RGB))
                else:
                    break

            if preprocess_pytorch is not None:
                all_tensors_l = torch.stack(frame_l)
                yield {"Tensors": all_tensors_l, "Images": frames_orig, "ShotInfo": shot}
            else:
                yield {"Tensors": None, "Images": frames_orig, "ShotInfo": shot}

        cap.release()

    def getShotFromID(self, sid=-1):
        for shot in self.shot_list:
            if(shot.sid == sid):
                return shot
        return None

    def export2csv(self, filepath=None):

        printCustom(f"Exporting results to CSV \"{filepath}\'", type=STDOUT_TYPE.INFO)

        if(filepath == None):
            print("ERROR: You have to specify a vailid path! csv export aborted!")
            exit()

        ## clean up
        remove_csv_export(filepath)

        ## export
        field_names = ['movie_name', 'sid', 'start', 'stop', 'fid', 'oid', 'bb_x1', 'bb_y1', 'bb_x2', 'bb_y2',
                       'object_conf', 'class_score', 'class_name']

        add_header(filepath, field_names)
        for shot in self.shot_list:
            dict_l = shot.convertObjectList2Dict()
            for dict_entry in dict_l:
                append_dict_as_row(filepath, dict_entry, field_names)

    def loadCsvExport(self, filepath="/data/share/maxrecall_vhh_mmsi/develop/videos/results/vhh_od/raw_results/test.csv"):
        print("load csv results export ... ")

        field_names =  ['movie_name', 'sid', 'start', 'stop', 'fid', 'oid', 'bb_x1', 'bb_y1', 'bb_x2', 'bb_y2',
                       'object_conf', 'class_score', 'class_name']

        dict_l = load_csv_as_dict(file_name=filepath, field_names=field_names)
        print(dict_l)

        self.shot_list = []

    def visualizeShotsWithBB(self, path=None, sid=-1, all_frames_tensors=None, boundingbox_flag=True,
               save_single_plots_flag=True, plot_flag=False, save_as_video_flag=True):
        #print("plot shot with bounding boxes ... ")

        if(path == None):
            print("Error: you need to specify a valid path!")
            exit()

        shot = self.getShotFromID(sid=sid)
        if(shot == None):
            print("ERROR: shot with specified ID [" + str(sid) + "] is not available!")
            return -1

        shot_frames_tensors = all_frames_tensors[shot.start_pos:shot.end_pos+1, :, :, :]
        frameSize = (int(shot_frames_tensors[0].size()[1]), int(shot_frames_tensors[0].size()[2]))

        video_results_path = path + str(self.vidName.split('.')[0]) + "/"

        if not os.path.exists(video_results_path):
            os.makedirs(video_results_path)

        if (save_single_plots_flag == True):
            if not os.path.exists(video_results_path + str(sid)):
                os.makedirs(video_results_path + str(sid))

        if (save_as_video_flag == True):
            out = cv2.VideoWriter(video_results_path + str(sid) + ".avi", cv2.VideoWriter_fourcc(*"MJPG"), 12, frameSize)

        for i in range(0, len(shot_frames_tensors)):
            frame_id = i + shot.start_pos

            frame = shot_frames_tensors[i]
            frame_np = np.array(frame).transpose((1, 2, 0))
            normalized_frame = frame_np.copy()
            normalized_frame = cv2.normalize(frame_np, normalized_frame, 0, 255, cv2.NORM_MINMAX)
            normalized_frame = normalized_frame.astype('uint8')

            if (boundingbox_flag == True):
                for obj in shot.object_list:
                    if (frame_id == obj.fid):
                        cv2.rectangle(normalized_frame,  (obj.bb_x1, obj.bb_y1),  (obj.bb_x2, obj.bb_y2), (0, 255, 0), 1)
                        font = cv2.FONT_HERSHEY_SIMPLEX
                        bottomLeftCornerOfText = (int(obj.bb_x1), int(obj.bb_y1) - 3)
                        fontScale = 0.4
                        fontColor = (0, 255, 0, 255)
                        thickness = 1
                        line_type = cv2.LINE_AA
                        obj_text = str(obj.object_class_name) + ": " + str(round(obj.object_conf, 2))
                        cv2.putText(normalized_frame,
                                    obj_text,
                                    bottomLeftCornerOfText,
                                    font,
                                    fontScale,
                                    fontColor,
                                    thickness,
                                    line_type)

            if (save_as_video_flag == True):
                out.write(normalized_frame)

            if (save_single_plots_flag == True):
                cv2.imwrite(video_results_path + str(sid) + "/" + str(frame_id) + ".png", normalized_frame)

            if (plot_flag == True):
                cv2.imshow("Shot ID:" + str(sid), normalized_frame)
                cv2.waitKey(10)

        if (save_as_video_flag == True):
            out.release()

        if (plot_flag == True):
            cv2.destroyAllWindows()




import numpy as np
import torch

from .deep.feature_extractor import Extractor
from .sort.nn_matching import NearestNeighborDistanceMetric
from .sort.preprocessing import non_max_suppression
from .sort.detection import Detection
from .sort.tracker import Tracker


__all__ = ['DeepSort']


class DeepSort(object):
    def __init__(self, model_path, max_dist=0.2, min_confidence=0.3, nms_max_overlap=0.5, max_iou_distance=0.7, max_age=70, n_init=3, use_cuda=True):

        self.min_confidence = min_confidence
        self.nms_max_overlap = nms_max_overlap

        self.extractor = Extractor(model_path, use_cuda=use_cuda)

        max_cosine_distance = max_dist
        nn_budget = 100
        metric = NearestNeighborDistanceMetric("cosine", max_cosine_distance, nn_budget)
        self.tracker = Tracker(metric, max_iou_distance=max_iou_distance, max_age=max_age, n_init=n_init)

    def reset(self):
        self.tracker.reset()

    def update(self, bbox_xywh, confidences, classes, ori_img):
        self.height, self.width = ori_img.shape[:2]
        # generate detections
        features = self._get_features(bbox_xywh, ori_img)
        bbox_tlwh = self._xywh_to_tlwh(bbox_xywh)
        detections = [Detection(bbox_tlwh[i], conf, classes[i], features[i]) for i,conf in enumerate(confidences) if conf>self.min_confidence]

        # run on non-maximum supression
        boxes = np.array([d.tlwh for d in detections])
        scores = np.array([d.confidence for d in detections])
        indices = non_max_suppression(boxes, self.nms_max_overlap, scores)
        detections = [detections[i] for i in indices]

        # update tracker
        self.tracker.predict()
        self.tracker.update(detections)

        # output bbox identities
        outputs = []
        for track in self.tracker.tracks:
            if not track.is_confirmed() or track.time_since_update > 1:
                continue
            box = track.to_tlwh()
            x1,y1,x2,y2 = self._tlwh_to_xyxy(box)
            track_id = track.track_id
            track_class = track.class_idx
            outputs.append(np.array([x1,y1,x2,y2,track_id,track_class], dtype=np.int))
        if len(outputs) > 0:
            outputs = np.stack(outputs,axis=0)
        return outputs


    """
    TODO:
        Convert bbox from xc_yc_w_h to xtl_ytl_w_h
    Thanks JieChen91@github.com for reporting this bug!
    """
    @staticmethod
    def _xywh_to_tlwh(bbox_xywh):
        if isinstance(bbox_xywh, np.ndarray):
            bbox_tlwh = bbox_xywh.copy()
        elif isinstance(bbox_xywh, torch.Tensor):
            bbox_tlwh = bbox_xywh.clone()
        bbox_tlwh[:,0] = bbox_xywh[:,0] - bbox_xywh[:,2]/2.
        bbox_tlwh[:,1] = bbox_xywh[:,1] - bbox_xywh[:,3]/2.
        return bbox_tlwh


    def _xywh_to_xyxy(self, bbox_xywh):
        x,y,w,h = bbox_xywh
        x1 = max(int(x-w/2),0)
        x2 = min(int(x+w/2),self.width-1)
        y1 = max(int(y-h/2),0)
        y2 = min(int(y+h/2),self.height-1)
        return x1,y1,x2,y2

    def _tlwh_to_xyxy(self, bbox_tlwh):
        """
        TODO:
            Convert bbox from xtl_ytl_w_h to xc_yc_w_h
        Thanks JieChen91@github.com for reporting this bug!
        """
        x,y,w,h = bbox_tlwh
        x1 = max(int(x),0)
        x2 = min(int(x+w),self.width-1)
        y1 = max(int(y),0)
        y2 = min(int(y+h),self.height-1)
        return x1,y1,x2,y2

    def _xyxy_to_tlwh(self, bbox_xyxy):
        x1,y1,x2,y2 = bbox_xyxy

        t = x1
        l = y1
        w = int(x2-x1)
        h = int(y2-y1)
        return t,l,w,h
    
    def _get_features(self, bbox_xywh, ori_img):
        im_crops = []
        for box in bbox_xywh:
            x1,y1,x2,y2 = self._xywh_to_xyxy(box)
            im = ori_img[y1:y2,x1:x2]
            im_crops.append(im)
        if im_crops:
            features = self.extractor(im_crops)
        else:
            features = np.array([])
        return features





from .deep_sort import DeepSort


__all__ = ['DeepSort', 'build_tracker']


def build_tracker(cfg, use_cuda):
    return DeepSort(cfg.DEEPSORT.REID_CKPT, 
                max_dist=cfg.DEEPSORT.MAX_DIST, min_confidence=cfg.DEEPSORT.MIN_CONFIDENCE, 
                nms_max_overlap=cfg.DEEPSORT.NMS_MAX_OVERLAP, max_iou_distance=cfg.DEEPSORT.MAX_IOU_DISTANCE, 
                max_age=cfg.DEEPSORT.MAX_AGE, n_init=cfg.DEEPSORT.N_INIT, nn_budget=cfg.DEEPSORT.NN_BUDGET, use_cuda=use_cuda)
    












import os
from pathlib import Path
import cv2
import numpy as np
import torch
from torchvision import models, transforms
from torch import nn
from torch.utils.data import Dataset
import wandb
import random
import sklearn.metrics
import yaml

path_to_config = "config/config_vhh_od.yaml"
batchsize = 8
std = 55.7495 / 255
mean = 87.2851 / 255

fp = open(path_to_config, 'r')
config = yaml.load(fp, Loader=yaml.BaseLoader)
used_classes = config['OdCore']['CLASSES_FOR_CLASSIFIER']
n_classes = len(used_classes)

transform_normalize = transforms.Normalize(mean=[mean, mean, mean], std=[std, std, std])

test_transform =  transforms.Compose([
        transforms.ToTensor(),
        transforms.Resize((224,224)),
        transforms.Grayscale(num_output_channels=3),
        # Normalization from our data:
        transform_normalize,
    ])

def class_to_idx(class_name):
    return used_classes.index(class_name)

def idx_to_class(idx):
    return used_classes[idx]

def get_class_distribution(dataset):
    samples = [0 for i in used_classes]
    for i, sample in enumerate(dataset):
        label = sample["class"]
        samples[label] += 1

    print([float(i) for i in samples])
    return [float(i)/len(dataset) for i in samples]

def output_to_prediction(outputs):
    return [np.argmax(outputs[i]) for i in range(outputs.shape[0])]

def get_metrics(outputs, labels, do_class_metrics = False):
    precision, recall, f1, _ = sklearn.metrics.precision_recall_fscore_support(labels, outputs, average="macro")
    metrics = {"macro_prec": precision, "macro_rec": recall, "macro_f1": f1}

    if do_class_metrics:
        for label in range(n_classes):
            precision, recall, f1, _ = sklearn.metrics.precision_recall_fscore_support(labels, outputs, average="macro", labels = [label])
            metrics["prec ({0})".format( idx_to_class(label))] = precision
            metrics["rec ({0})".format(idx_to_class(label))] = recall
            metrics["f1 ({0})".format(idx_to_class(label))] = f1

    nr_correct_predictions = sum([1 for i in range(len(outputs)) if outputs[i] == labels[i]])
    accuracy = float(nr_correct_predictions) / len(outputs)
    metrics["accuracy"] = accuracy
    return metrics

class Classifier():
    """
    Classifies an image of a person
    """
    def __init__(self, model_name, path_to_weights, device):
        self.model = ClassifierModel(model_name)
        self.model.load_state_dict(torch.load(path_to_weights))
        self.device = device
        self.model.to(device)

    def inference(self, crops):
        """
        Takes a batch of (unprocessed) crops and classifies them
        """
        self.model.eval()
        with torch.no_grad():
            crops = crops.to(self.device)
            outputs = self.model(crops).cpu().detach().numpy()
            class_idx = output_to_prediction(outputs)
            class_names = [idx_to_class(idx) for idx in class_idx]
        return class_names, class_idx

    def to(self, device):
        self.model.to(device)
        self.device = device

    def inference_from_img(self, img):
        """
        Takes a batch of (unprocessed) crops and returns scores
        """
        return self.inference(img)
    

class ClassifierModel(nn.Module):
    """
    A wrapper around a pretrained classifier model
    Classifies a batch of crops according to used_classes
    """

    def __init__(self, model_name):
        super(ClassifierModel, self).__init__()

        if model_name == "densenet121":
            self.model = models.densenet121(pretrained=True)
            num_ftrs = self.model.classifier.in_features
            self.model.classifier = nn.Linear(num_ftrs, n_classes)
        elif model_name == "vgg11":
            self.model = models.vgg11(pretrained=True)
            num_ftrs = self.model.classifier[6].in_features
            self.model.classifier[6] = nn.Linear(num_ftrs,n_classes)
        elif model_name == "vgg16":
            self.model = models.vgg16(pretrained=True)
            num_ftrs = self.model.classifier[6].in_features
            self.model.classifier[6] = nn.Linear(num_ftrs,n_classes)
        elif model_name == "wide_resnet50_2":
            self.model = models.wide_resnet50_2(pretrained=True)
            self.model.fc = nn.Linear(2048, n_classes)
        self.input_size = 224

    def forward(self, x):
        return self.model(x)

class PersonsDataset(Dataset):
    def __init__(self, img_folder, shuffle=True, transforms = None, dropFilesFrom = None):
        """
        Args:
            img_folder (string): The folder containing the class folders
            dropFilesFrom (list of tuples (string, int)): 
                Can be used to downsample or upsample data. 
                [("corpse", 5)] would mean that only every fifth image from the corpse folder would be used.
                [("corpse", 5), ("soldier", -3)] means the same as above and that every image from the soldier folder will be used three times. 
        """
        self.img_folder = img_folder
        self.transform = transforms

        self.data = []


        do_drop_classes = dropFilesFrom is not None
        if do_drop_classes:
            dropped_classes, acceptance_mod = list(zip(*dropFilesFrom))
            print(dropped_classes, acceptance_mod)


        # Load data
        for directory in os.listdir(img_folder):
            if directory in used_classes:
                for i, path in enumerate(Path(os.path.join(img_folder, directory)).rglob('*.png')):
                    # Up and downsampling
                    if do_drop_classes and directory in dropped_classes:
                        acceptance = acceptance_mod[dropped_classes.index(directory)] 
                        # Drop samples if acceptance > 0 (downsampling)
                        if acceptance > 0 and i % acceptance != 0:
                            continue
                        # Import samples multiple times if acceptance < 0 (upsampling)
                        elif acceptance < 0:
                            for j in range(int(-acceptance)):
                                self.data.append((str(path), class_to_idx(directory)))

                    self.data.append((str(path), class_to_idx(directory)))

        # Shuffle
        if shuffle:
            np.random.shuffle(self.data)

    def __len__(self):
        return len(self.data)

    def __getitem__(self, idx):
        img_path = self.data[idx][0]
        img_class = self.data[idx][1]
        img =  cv2.imread(img_path)
        np_img = np.asarray(img)

        if self.transform:
            np_img = self.transform(np_img)

        return {"image": np_img, "class": img_class}

def evaluate(dataloader, name, epoch, device, model, criterion, do_class_metrics = False):
    model.eval()
    with torch.no_grad():
        curr_loss = 0
        predictions_train = []
        labels_train = []
        for batch in dataloader:
            inputs = batch["image"].to(device)
            labels = batch["class"].to(device)
            outputs = model(inputs)

            labels_train += labels.cpu().detach().numpy().tolist()
            predictions_train += output_to_prediction(outputs.cpu().detach().numpy())

            loss = criterion(outputs, labels)
            curr_loss += loss.cpu().item()

        metrics = get_metrics(predictions_train, labels_train, do_class_metrics=do_class_metrics)
        curr_loss = curr_loss / len(dataloader)
        metrics["total_loss"] = curr_loss
        print("\t{0} loss: {1}\n\t{0} accuracy: {2}".format(name, curr_loss, metrics["accuracy"]))
        return metrics


class obj_list_loader:
    """
    This is a generator that allows you to loop through custom_objects and images
    """

    def __init__(self, custom_obj_list, relevant_frames):
        self.custom_obj_list = custom_obj_list
        self.relevant_frames = relevant_frames

    def loop(self):
        crops = None
        indices = []

        curr_frame = -1
        curr_frame_id = -1

        # Indices of objects in which persons appear
        for i, obj in enumerate(self.custom_obj_list):

            # None means that we did not find predictions for this frame
            if obj is None:
                curr_frame += 1
                continue

            # If an object is from a new frame then increase our frame counter
            if obj.fid > curr_frame_id:
                    curr_frame += 1
                    curr_frame_id = obj.fid

            if obj.object_class_name == "person":
                rel_image = self.relevant_frames[curr_frame]

                # Sometimes OD predicts boundary boxes with coordinates < 0, cannot crop those so ignore them
                if(obj.bb_y1 < 0) or (obj.bb_y2 < 0) or (obj.bb_x1 < 0) or (obj.bb_x2 < 0) or (obj.bb_y2 <= obj.bb_y1) or (obj.bb_x2 <= obj.bb_x1) \
                or (obj.bb_y2 >= rel_image.shape[0]) or (obj.bb_x2 >= rel_image.shape[1]):
                    continue
                
                indices.append(i)
                image = rel_image[obj.bb_y1:obj.bb_y2, obj.bb_x1:obj.bb_x2]
                image = test_transform(image)

                # Add a new dimenions in front on which we will append the different images
                image = torch.unsqueeze(image, 0)

                if crops is not None:
                    crops = torch.cat((crops, image), 0)
                else:
                    crops = image
            else:
                continue

            if len(indices) == batchsize:
                yield crops, indices
                crops = None
                indices = []
        yield crops, indices




def run_classifier_on_list_of_custom_objects(classifier, custom_obj_list, relevant_frames):
    generator = obj_list_loader(custom_obj_list, relevant_frames)

    for crops, indices in generator.loop():
        # No crops means nothing to classify
        if crops is None:
            return

        crops = torch.split(crops, batchsize, dim=0)
        idx = 0
        for batch in crops:
            class_names, _ = classifier.inference_from_img(batch)
            
            for class_name in class_names:
                custom_obj_list[indices[idx]].add_person_classification(class_name)
                idx += 1



import torch

features = torch.load("features.pth")
qf = features["qf"]
ql = features["ql"]
gf = features["gf"]
gl = features["gl"]

scores = qf.mm(gf.t())
res = scores.topk(5, dim=1)[1][:,0]
top1correct = gl[res].eq(ql).sum().item()

print("Acc top1:{:.3f}".format(top1correct/ql.size(0)))


